<?php

//
// Short code `ccom_fbt` displays Frequently Bought Together on the front-end single product pages.
//

// Hook Into Single Products
add_action( 'woocommerce_after_single_product_summary', function() {

	// Determine Cross Sells Default
	$ccom_fbt_cross_sells = get_option( 'ccom_fbt_cross_sells', true )
		? get_option( 'ccom_fbt_cross_sells', true ) : 'no';

	// Output
	echo do_shortcode(
		sprintf(
			'[ccom_fbt cross_sells="%s"]',
			$ccom_fbt_cross_sells
		)
	);

} );

// Add Shortcode
add_shortcode( 'ccom_fbt', function( $atts ) {

	// Setup Default Settings
	$atts = shortcode_atts(
		[
			'cross_sells' => 'no',
			'heading' => __(
				'Frequently purchased together&hellip;',
				'ccom-upsell-trio'
			),
			'heading_tag' => 'h2',
		], $atts, 'ccom_fbt'
	);

	// Front End Or Bail
	if( is_admin() ) {
		return;
	}

	// Coupons Must Be Enabled In WooCommerce
	if( ! get_option( 'woocommerce_enable_coupons' ) === 'yes' ) {
		return;
	}

	// Woo Product Or Bail
	global $product;
	if( ! $product || ! method_exists( $product, 'is_purchasable' ) ) {
		return;
	}

	// Must Be Published And In Stock
	if( ! $product->is_purchasable() ) {
		return;
	}

	// Maybe Front Pad With Cross Sells
	$priority_one = $atts['cross_sells'] === 'yes'
		? $product->get_cross_sell_ids()
		: [];

	// Get Cross-Sells And Recommended Product IDs
	$recommendations = array_merge(

		// Priority One: Cross-Sells
		(array) $priority_one,

		// Priority Two: Our Data
		(array) $product->get_meta( 'ccom_recommendations', true ),

		// Priority Three: End Pad WooCommerce Defaults
		(array) wc_get_related_products( $product->get_id(), 5 )

	);

	// Bail When No Recommendations Found
	if( ! $recommendations ) {
		return;
	}

	// Remove Any Non Purchaseable Recommendations
	foreach( $recommendations as $i => $rec_product_id ) {
		$rec_product = wc_get_product( $rec_product_id );
		if(
			! $rec_product
			|| ! method_exists( $rec_product, 'is_purchasable' )
			|| ! method_exists( $rec_product, 'is_in_stock' )
			|| ! method_exists( $rec_product, 'is_type' )
		) {
			unset( $recommendations[$i] );
			continue;
		}
		if( ! $rec_product->is_purchasable() || ! $rec_product->is_in_stock() ) {
			unset( $recommendations[$i] );
			continue;
		}
	}

	// Bail When No Recommendations Remain
	if( ! $recommendations ) {
		return;
	}

	// Start Output Accumulator
	$output = [];

	// Current Product Output
	$output[] = sprintf(
		'
			<li class="product" data-price="%s">
				%s<br>%s: %s
				<input type="hidden" id="ccom_fbt_self_id" name="ccom_fbt_a2c[]" value="%d">
			</li>
		',
		$product->get_price(),
		$product->get_image( 'thumbnail', [ 'style' => 'display:inline-block;' ] ),
		__( 'This product', 'ccom-upsell-trio' ),
		$product->get_name(),
		$product->get_id()
	);

	// Unique Recommendations Only
	$recommendations = array_unique( $recommendations );

	// Cut To Top Three Recommendations
	$recommendations = array_slice( $recommendations, 0, 3 );

	// Loop Recommended Products
	foreach( $recommendations as $rec_product_id ) {
		$rec_product = wc_get_product( $rec_product_id );

		// Variable Product Maybe Swap To Default Variation
		if( $rec_product->is_type( 'variable' ) ) {
			$default_attributes = $rec_product->get_default_attributes();
			if( $default_attributes ) {
				$prefixed_slugs = array_map(
					function( $pa_name ) {
						return 'attribute_'. $pa_name;
					}, array_keys( $default_attributes )
				);
				$default_variation_id = ( new \WC_Product_Data_Store_CPT() )->find_matching_product_variation(
					$rec_product, array_combine( $prefixed_slugs, $default_attributes )
				);
				if( $default_variation_id ) {
					$rec_product_id = $default_variation_id;
					$rec_product = wc_get_product( $default_variation_id );
				}
			}
		}

		// Accumulate Recommended Product Markup
		$output[] = sprintf(
			'
				<li class="product" data-price="%s">
					<a href="%s">
						%s
					</a>
					<br>+ %s
					<br>%s
					<br><mark style="padding:0.25em 1em;line-height:2em;">
						<input type="checkbox" name="ccom_fbt_a2c[]" value="%d"
							aria-label="Add to cart" checked="checked">
					</mark>
				</li>
			',
			$rec_product->get_price(),
			$rec_product->get_permalink(),
			$rec_product->get_image( 'thumbnail', [ 'style' => 'display:inline-block;' ] ),
			$rec_product->get_name(),
			$rec_product->get_price_html(),
			$rec_product_id
		);

	}

	// Output Form And Row Of Products
	ob_start();
	?>

	<form method="POST" id="ccom_fbt_form"
		action="<?php echo wc_get_cart_url(); ?>"
		style="clear:both;padding-top:1em;">

		<<?php echo esc_html( $atts['heading_tag'] ); ?>>
			<?php echo esc_html( $atts['heading'] ); ?>
		</<?php echo esc_html( $atts['heading_tag'] ); ?>>

		<ul class="ccom_fbt_row"
			style="text-align:center;display:grid;margin:0;list-style:none;grid-template-columns:repeat(auto-fit,minmax(200px,1fr));grid-column-gap:1em;grid-auto-flow:dense;">
			<?php echo implode( "\n", array_filter( $output, 'wp_kses_post' ) ); ?>
		</ul>

		<div id="ccom_fbt_totals" style="text-align:center;">

			<p>
				<?php _e( 'Price for selected combo:', 'ccom-upsell-trio' ); ?>
				<span class="woocommerce-Price-amount amount">
					<bdi>
						<span class="woocommerce-Price-currencySymbol">$</span><span
							id="ccom_fbt_total" style="text-decoration:underline;"></span>
					</bdi>
				</span>
				&nbsp;

				<span class="woocommerce-Price-amount amount">
					<bdi>
						<span class="woocommerce-Price-currencySymbol">$</span><span
							id="ccom_fbt_subtotal" style="text-decoration:line-through;"></span>
					</bdi>
				</span>
				&nbsp;

				<strong>
					Save
					<span class="woocommerce-Price-amount amount">
						<bdi>
							<span class="woocommerce-Price-currencySymbol">$</span><span
								id="ccom_fbt_discount"></span>
						</bdi>
					</span>
					(10% off)
				</strong>
			</p>

			<p>
				<button type="submit" class="ccom_fbt_add_to_cart wp-block-button__link">
					<?php _e( 'Add Selected Combo to Cart', 'ccom-upsell-trio' ); ?>
				</button>
			</p>

		</div>

	</form>

	<?php
	return ob_get_clean();

} );